/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bos.jobs;

import java.util.Arrays;
import java.util.List;

import de.frankmuenster.mahoe.banks.bos.BankOfScotland;
import de.frankmuenster.mahoe.banks.bos.BosUtils;
import de.frankmuenster.mahoe.banks.bos.web.BosBankMessages;
import de.frankmuenster.mahoe.banks.bos.web.BosWebAccountInformation;
import de.frankmuenster.mahoe.banks.bos.web.BosWebSite;
import de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.PasswdCache;
import de.frankmuenster.mahoe.utils.TransactionSaver;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementation of the Bank of Scotland account synchronisation
 *
 * @author Frank Mnster
 */
public class BosSynchronizeJobKontoauszug
        extends AbstractMahoeSynchronizeJobKontoauszug<BosWebAccountInformation, BosBankMessages> {

    /**
     * Standard constructor
     */
    public BosSynchronizeJobKontoauszug() {

        super();
    }

    /**
     * @see de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug#getWebSite()
     */
    @Override
    protected void getWebSite() throws WebSiteException, ApplicationException {

        BosUtils.checkSelenium();
        this.webSite = BosWebSite.getInstance(this.konto);

    }

    /**
     * @see de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug#handleTransactions()
     */
    @Override
    protected void handleTransactions() throws WebSiteException, ApplicationException {

        this.monitor.setStatusText(
                I18N.tr("SynchronizeJobKontoauszug.getBooked", this.accountInfo.getLastTransactionDateAsString())); //$NON-NLS-1$
        final List<Umsatz> booked = this.webSite.getBookedTransactions();

        this.monitor.setPercentComplete(40);

        // Debug-Ausgabe
        this.debugLogTransactions(booked);

        // Speicher gebuchte Umstze
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savingBooked")); //$NON-NLS-1$
        final int bookedCount = TransactionSaver.saveBooked(this.konto, booked);
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$
        this.monitor.setPercentComplete(50);
        Logger.info(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$

    }

    /**
     * @see de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug#login()
     */
    @Override
    protected void login() throws WebSiteException, ApplicationException {

        final char[] password = PasswdCache.getPassword(BankOfScotland.NAME, this.kundennummer);
        try {
            this.webSite.login(password);
        } catch (final WebSiteException e) {
            // Passwort zurcksetzten
            PasswdCache.revoke(BankOfScotland.NAME, this.kundennummer);
            throw e;
        }
        Arrays.fill(password, ' ');

    }

}
