
package de.frankmuenster.mahoe.banks.bsh.json;

import java.math.BigDecimal;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.mahoe.banks.bsh.json.deserializer.MoneyDeserializer;

/**
 * JSON Account Object (ein Vertrag)
 *
 * @author Frank Mnster/Quirin Huber
 */
@JsonIgnoreProperties(ignoreUnknown = true)
public class Account {

    /**
     * Laufendes Saldo
     */
    @JsonDeserialize(using = MoneyDeserializer.class)
    private BigDecimal accountBalance;

    /**
     * Die Bausparnummer
     */
    @JsonProperty
    private String bsn;

    /**
     * Der Vertragsbuchstabe
     */
    @JsonProperty("contractletter")
    private String contractLetter;

    /**
     * Die Vertrags-Suffixnummer
     */
    @JsonProperty("contractnumber")
    private String contractNumber;

    /**
     * Die Vertragsnummer
     */
    @JsonProperty("coreid")
    private String coreId;

    /**
     * Der Vertagstyp
     */
    @JsonProperty
    private String type;

    /**
     * Die BVTN
     */
    @JsonProperty
    private String vtn;

    /**
     * Standardkonstruktor
     */
    public Account() {
	// Absichtlich leer
    }

    /**
     * Liefert das laufende Saldo
     *
     * @return das laufende Saldo
     */
    public BigDecimal getAccountBalance() {

	return this.accountBalance;
    }

    /**
     * Liefert die BSN
     *
     * @return die BSN
     */
    public String getBsn() {

	return this.bsn;
    }

    /**
     * Liefert die BSN und den Typen
     *
     * @return die BSN und den Typen
     */
    public String getBsnType() {

	return this.bsn + " - " + this.type; //$NON-NLS-1$
    }

    /**
     * Liefert den Vertagsbuchstaben
     *
     * @return den Vertagsbuchstaben
     */
    public String getContractLetter() {

	return this.contractLetter;
    }

    /**
     * Liefert die Vertrags-Suffixnummer
     *
     * @return die Vertrags-Suffixnummer
     */
    public String getContractNumber() {

	return this.contractNumber;
    }

    /**
     * Liefert die Vertragshauptnummer
     *
     * @return die Vertragshauptnummer
     */
    public String getCoreId() {

	return this.coreId;
    }

    /**
     * Liefert den Vertragstyp
     *
     * @return den Vertragstyp
     */
    public String getType() {

	return this.type;
    }

    /**
     * Liefert die VTN
     *
     * @return die VTN
     */
    public String getVtn() {

	return this.vtn;
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {

	return String.format(
		"Account [accountBalance=%s, bsn=%s, contractLetter=%s, contractNumber=%s, coreId=%s, type=%s, vtn=%s]", //$NON-NLS-1$
		this.accountBalance, this.bsn, this.contractLetter, this.contractNumber, this.coreId, this.type,
		this.vtn);
    }
}
