/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bsh.json.accountdetails;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * Vertragdetails Umsatzmonat
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "name", "fullname", "entries" })
public class TransactionMonth { // NOPMD

    /**
     * Map with 2 char month name to integer relation
     */
    private static final Map<String, Integer> month = new HashMap<>();  // NOPMD
    static {
        TransactionMonth.month.put("JAN", 1); //$NON-NLS-1$
        TransactionMonth.month.put("FEB", 2); //$NON-NLS-1$
        TransactionMonth.month.put("MRZ", 3); //$NON-NLS-1$
        TransactionMonth.month.put("APR", 4); //$NON-NLS-1$
        TransactionMonth.month.put("MAI", 5); //$NON-NLS-1$
        TransactionMonth.month.put("JUN", 6); //$NON-NLS-1$
        TransactionMonth.month.put("JUL", 7); //$NON-NLS-1$
        TransactionMonth.month.put("AUG", 8); //$NON-NLS-1$
        TransactionMonth.month.put("SEP", 9); //$NON-NLS-1$
        TransactionMonth.month.put("OKT", 10); //$NON-NLS-1$
        TransactionMonth.month.put("NOV", 11); //$NON-NLS-1$
        TransactionMonth.month.put("DEZ", 12); //$NON-NLS-1$
    }

    /**
     * Kurzform Monat
     */
    @JsonProperty("name")
    private String name;

    /**
     * Langform Monat
     */
    @JsonProperty("fullname")
    private String fullname;

    /**
     * Buchungen im Monat
     */
    @JsonProperty("entries")
    private List<TransactionEntry> entries;

    /**
     * Standardkonstruktor
     */
    public TransactionMonth() {
        // Absichtlich leer
    }

    /**
     * Liefert ein Liste mit dem Buchen in diesem Monat.
     *
     * @return ein Liste mit dem Buchen des Monats.
     */
    @JsonProperty("entries")
    public List<TransactionEntry> getEntries() {

        return this.entries;
    }

    /**
     * Liefert die Langform des Monats
     *
     * @return die Langform des Monats
     */
    @JsonProperty("fullname")
    public String getFullname() {

        return this.fullname;
    }

    /**
     * Liefert den Monat als zweistellige Zahl.
     *
     * @return den Monat als zweistellige Zahl.
     */
    public int getMonthAsInt() { // NOPMD

        int iMonat = 1;
        final String monthName = this.name.toUpperCase(Locale.GERMAN);
        if (TransactionMonth.month.containsKey(monthName)) {
            iMonat = TransactionMonth.month.get(monthName);
        }
        return iMonat;

    }

    /**
     * Liefert die Kurzform des Monats
     *
     * @return die Kurzform des Monats
     */
    @JsonProperty("name")
    public String getName() {

        return this.name;
    }

}
