
package de.frankmuenster.mahoe.banks.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.mahoe.banks.bsh.json.deserializer.MoneyDeserializer;

/**
 * Vertragdetails - Umsatz
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "year", "saldo", "month" })
public class TransactionYear {

    /**
     * Das Jahr der Buchungen
     */
    @JsonProperty("year")
    private Integer year;

    /**
     * Das Endsaldo in dem Jahr
     */
    @JsonProperty("saldo")
    @JsonDeserialize(using = MoneyDeserializer.class)
    private BigDecimal saldo;

    /**
     * Liste der Monate
     */
    @JsonProperty("month")
    private List<TransactionMonth> month;

    /**
     * Standardkonstruktor
     */
    public TransactionYear() {
	// Absichlich leer
    }

    /**
     * Liefert eine Liste aller Monate mit Umstzen.
     *
     * @return die Liste aller Monate mit Umstzen.
     */
    @JsonProperty("month")
    public List<TransactionMonth> getMonth() {

	return this.month;
    }

    /**
     * Liefert das Endsaldo des Jahres
     *
     * @return das Endsaldo
     */
    @JsonProperty("saldo")
    public BigDecimal getSaldo() {

	return this.saldo;
    }

    /**
     * Liefert das Jahr
     *
     * @return das Jahr
     */
    @JsonProperty("year")
    public Integer getYear() {

	return this.year;
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {

	return String.format("TransactionYear [year=%s, saldo=%s, month=%s]", this.year, this.saldo, this.month); //$NON-NLS-1$
    }

}
