/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web; // NOPMD

import java.io.IOException;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.NicelyResynchronizingAjaxController;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.WebClientOptions;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.DomNodeList;
import com.gargoylesoftware.htmlunit.html.HtmlElement;
import com.gargoylesoftware.htmlunit.html.HtmlForm;
import com.gargoylesoftware.htmlunit.html.HtmlInput;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTableBody;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.statements.AbstractAccountStatementFinder;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.AbstractWebSite;
import de.frankmuenster.mahoe.web.MahoeKontoauszugPDF;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Stellt das Webseiten-Interface dar.
 *
 * @author Frank Mnster
 */
public class TargoBankWebSite extends AbstractWebSite<TargoBankWebAccountInformation, TargoBankMessages> {

    /**
    *
    */
    private static final String I18N_LOGOFF_FAILED = "TargoBankWebSite.logoffFailed"; //$NON-NLS-1$

    /**
     * Seite mit den PDF-Abrechnungen nicht gefunden
     */
    protected static final String PDFSITE_NOT_FOUND = "Abrechnungen konnten nicht ge\u00F6ffnet werden"; //$NON-NLS-1$

    /**
     * Unsere BceeWebSite instance
     */
    private static TargoBankWebSite webSite;

    /**
     * Das Konto
     */
    private final Konto konto;

    /**
     * Das Implementierungs-Objekt fr die Umstze
     */
    private final TargoBankWebSiteTransactionsImpl transImpl;

    /**
     * Das Login/die Kundenkennung
     */
    private final String kundenkennung;

    /**
     * Liefert die Instance der WebSeite
     *
     * @param konto
     * @return die WebSeite
     * @throws WebSiteException
     * @throws ApplicationException
     */
    public static AbstractWebSite<TargoBankWebAccountInformation, TargoBankMessages> getInstance(final Konto konto)
            throws WebSiteException, ApplicationException {

        String cardAccount;
        String cardNumber;
        String cardCompany;
        String loginName;
        try {
            loginName = konto.getKundennummer();
            if (StringUtils.isEmpty(loginName)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.loginMissing")); //$NON-NLS-1$
            }
            cardAccount = konto.getMeta(MahoeSynchronizeBackend.CARD_ACCOUNT, null);
            if (StringUtils.isEmpty(cardAccount)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.cardAccountMissing")); //$NON-NLS-1$
            }
            cardNumber = konto.getMeta(MahoeSynchronizeBackend.CARD_NUMBER, null);
            if (StringUtils.isEmpty(cardNumber)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.cardNumberMissing")); //$NON-NLS-1$
            }
            cardNumber = cardNumber.replaceAll("\\s+", StringUtils.EMPTY); //$NON-NLS-1$
            cardCompany = konto.getMeta(MahoeSynchronizeBackend.CARD_COMPANY, null);
            if (StringUtils.isEmpty(cardCompany)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.cardCompanyMissing")); //$NON-NLS-1$
            }
            if (TargoBankWebSite.webSite == null) { // NOPMD
                TargoBankWebSite.webSite = new TargoBankWebSite(konto, loginName, cardCompany, cardAccount, cardNumber);
            } else {
                // Ist das Konto dasselbe
                if (TargoBankWebSite.webSite.konto.equals(konto)) {
                    // Aktualisiere Startdatum der Umsatzabfrage
                    final Date lastTransacDate = Utils.getLastUmsatzDate(konto,
                            TargoBankWebSite.webSite.getStartdate());
                    TargoBankWebSite.webSite.accountInfo.setLastTransactionDate(lastTransacDate);
                } else {
                    TargoBankWebSite.webSite.logoff();
                    TargoBankWebSite.webSite = new TargoBankWebSite(konto, loginName, cardCompany, cardAccount,
                            cardNumber);
                }
            }
        } catch (final RemoteException e) {
            final String msg = I18N.tr("TargoBankWebSite.readAccountInfoFailed"); //$NON-NLS-1$
            Logger.error(msg, e);
            throw new ApplicationException(msg, e);
        }
        return TargoBankWebSite.webSite;
    }

    /**
     * Erstellt ein neues Webseiten-Objekt zur Kontaktaufnahme mit der TargoBank.
     *
     * @param konto
     * @param login
     * @param cardCompany
     * @param cardAccount
     * @param cardNumber
     * @throws WebSiteException
     */
    public TargoBankWebSite(final Konto konto, final String login, final String cardCompany, final String cardAccount,
            final String cardNumber) throws WebSiteException {

        super(konto);
        this.konto = konto;
        this.kundenkennung = login;
        this.accountInfo.setCardCompany(cardCompany);
        this.accountInfo.setCardNumber(cardNumber);
        this.accountInfo.setCardAccount(cardAccount);
        this.transImpl = new TargoBankWebSiteTransactionsImpl(this.webClient, this.accountInfo);
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
     */
    @Override
    public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung)
            throws WebSiteException, ApplicationException {

        Logger.info(I18N.tr("TargoBankWebSite.transferNotSupported")); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getAccountInformation()
     */
    @Override
    public TargoBankWebAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

        try {
            final HtmlPage page = TargoBankWebSiteImpl.openAccount(this.webClient, this.accountInfo);
            final DomElement section = page.getElementById("page"); //$NON-NLS-1$
            final DomNodeList<HtmlElement> tables = section.getElementsByTagName(HtmlTable.TAG_NAME);
            final SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault()); //$NON-NLS-1$
            for (final HtmlElement element : tables) {
                if (element instanceof HtmlTable) {
                    final HtmlTable table = (HtmlTable) element;
                    Logger.trace("Table summery: " + table.getSummaryAttribute()); //$NON-NLS-1$
                    if (table.getSummaryAttribute().equals("Card Summary")) { //$NON-NLS-1$
                        // hole Grundinfos
                        String value;
                        final HtmlTableBody body = table.getBodies().get(0);
                        // Saldo
                        value = body.getRows().get(0).getCell(1).getTextContent().trim()
                                .replaceAll(AbstractWebSite.NON_BREAK_SPACE, StringUtils.EMPTY);
                        this.accountInfo.setSaldo(Utils.convertStringToBigDecimal(value));
                        // Verfgungsrahmen
                        value = body.getRows().get(1).getCell(1).getTextContent().trim()
                                .replaceAll(AbstractWebSite.NON_BREAK_SPACE, StringUtils.EMPTY);
                        this.accountInfo.setVerfuegungsrahmen(Utils.convertStringToDouble(value));
                        // Verfgbarer Betrag
                        value = body.getRows().get(2).getCell(1).getTextContent().trim()
                                .replaceAll(AbstractWebSite.NON_BREAK_SPACE, StringUtils.EMPTY);
                        this.accountInfo.setSaldoAvailable(Utils.convertStringToBigDecimal(value));
                        // Nchster Flligkeitstermin
                        value = body.getRows().get(2).getCell(3).getTextContent().trim()
                                .replaceAll(AbstractWebSite.NON_BREAK_SPACE, StringUtils.EMPTY);
                        final Date nextPayday = Utils.parseOrNull(sdf, value);
                        this.accountInfo.setNextPayDay(nextPayday);
                        Logger.info(I18N.tr("TargoBankWebSite.logSaldo") + this.accountInfo.getSaldo()); //$NON-NLS-1$
                        Logger.info(
                                I18N.tr("TargoBankWebSite.logSalsoAvailable") + this.accountInfo.getSaldoAvailable()); //$NON-NLS-1$
                        Logger.info(I18N.tr("TargoBankWebSite.logVerfuegungsrahmen") //$NON-NLS-1$
                                + this.accountInfo.getVerfuegungsrahmen());
                        Logger.info(I18N.tr("TargoBankWebSite.logPayDay") + this.accountInfo.getSaldo()); //$NON-NLS-1$
                        break;
                    }
                }
            }
        } catch (final WebSiteException e) {
            Logger.error(I18N.tr("TargoBankWebSite.readAccountOverviewFailedLog", e.getMessage()), e); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("TargoBankWebSite.readAccountOverviewFailed"), // NOPMD //$NON-NLS-1$
                    WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
        }
        return this.accountInfo;
    }

    @Override
    public List<MahoeKontoauszugPDF> getAccountSheets(final TargoBankWebAccountInformation accountInfo)
            throws WebSiteException {

        final AbstractAccountStatementFinder<?> accountSheets = new TargoBankAccountStatementFinder(this.webClient,
                accountInfo);
        accountSheets.findStatements();

        return accountSheets.getSheets();

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getBookedTransactions()
     */
    @Override
    public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

        return this.transImpl.getBookedTransactions();
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getPreBookedTransactions()
     */
    @Override
    public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {

        return this.transImpl.getPreBookTransactions();
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#isStillLoggedIn()
     */
    @Override
    public boolean isStillLoggedIn() throws WebSiteException {

        return false;
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#login(char[], String...)
     */
    @Override
    public void login(final char[] password, final String... loginParams) throws WebSiteException {

        if (loginParams.length != 0) {
            throw new WebSiteException(I18N.tr("TargoBankWebSite.loginParameterNumberWrong"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_ERROR);
        }
        try {
            final HtmlPage pageLogin = this.webClient.getPage(TargoBank.URL_LOGIN);
            Logger.trace(pageLogin.asText());
            final HtmlForm formLogin = pageLogin.getFormByName("bloc_ident"); //$NON-NLS-1$
            formLogin.getInputByName("_cm_user").setValueAttribute(this.kundenkennung); //$NON-NLS-1$
            formLogin.getInputByName("_cm_pwd").setValueAttribute(new String(password)); //$NON-NLS-1$
            final HtmlInput submitContinue = formLogin.getInputByName("submit"); //$NON-NLS-1$
            Logger.info(I18N.tr("TargoBankWebSite.sendLoginData")); //$NON-NLS-1$
            final HtmlPage pageLogin2 = submitContinue.click();
            Logger.debug("Seite: " + pageLogin2.asText()); //$NON-NLS-1$
            // Erster Test (Negativ Text ist vorhanden)
            if (pageLogin2.asText()
                    .contains("Die von Ihnen eingegebenen Daten sind falsch. Bitte versuchen Sie es erneut.")) { //$NON-NLS-1$
                Logger.warn(I18N.tr("TargoBankWebSite.loginDataWrong")); //$NON-NLS-1$
                throw new WebSiteException(I18N.tr("TargoBankWebSite.loginDataWrong"), //$NON-NLS-1$
                        WebSiteExceptionTypes.LOGIN_FAILED);
            }
            // Zweiter Test (positiv Text/Element ist nicht vorhanden)
            final DomElement logoutBtn = pageLogin2.getElementById("ei_tpl_ident_logout"); //$NON-NLS-1$
            if (logoutBtn == null) {
                Logger.warn(I18N.tr("TargoBankWebSite.logoutBtnNotFound")); //$NON-NLS-1$
                throw new WebSiteException(I18N.tr("TargoBankWebSite.logoutBtnNotFound"), //$NON-NLS-1$
                        WebSiteExceptionTypes.LOGIN_FAILED);
            }
        } catch (FailingHttpStatusCodeException | IOException e) {
            Logger.error(I18N.tr("TargoBankWebSite.loginFailedLog", e.getMessage())); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("TargoBankWebSite.loginFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_ERROR, e);
        }
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#logoff()
     */
    @Override
    public void logoff() throws WebSiteException {

        try {
            final HtmlPage logoff = this.webClient.getPage(TargoBank.URL_LOGOFF);
            if (logoff.asText().contains("Erfolgreich ausgeloggt.")) { //$NON-NLS-1$
                Logger.info(I18N.tr("TargoBankWebSite.logoffSuccessful")); //$NON-NLS-1$
            } else {
                throw new WebSiteException(I18N.tr(TargoBankWebSite.I18N_LOGOFF_FAILED),
                        WebSiteExceptionTypes.LOGOFF_FAILED);
            }
        } catch (FailingHttpStatusCodeException | IOException e) {
            Logger.error(I18N.tr(TargoBankWebSite.I18N_LOGOFF_FAILED), e);
            throw new WebSiteException(I18N.tr(TargoBankWebSite.I18N_LOGOFF_FAILED),
                    WebSiteExceptionTypes.LOGOFF_FAILED, e);
        }
    }

    /**
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getStartdate()
     */
    @Override
    protected Calendar getStartdate() {

        // Setzte das startDatum auf Januar 2016
        final Calendar startCalendar = Calendar.getInstance();
        startCalendar.set(2016, 0, 1);
        return startCalendar;
    }

    /**
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getWebClient()
     */
    @Override
    protected WebClient getWebClient() {

        final WebClient webcl = new WebClient();
        final WebClientOptions options = webcl.getOptions();
        options.setUseInsecureSSL(false);
        options.setJavaScriptEnabled(false);
        options.setThrowExceptionOnFailingStatusCode(false);
        options.setCssEnabled(true);
        options.setThrowExceptionOnScriptError(false);
        options.setRedirectEnabled(true);
        options.setPrintContentOnFailingStatusCode(true);
        webcl.setAjaxController(new NicelyResynchronizingAjaxController());
        webcl.getCookieManager().setCookiesEnabled(true);
        Utils.setProxyCfg(webcl, TargoBank.URL_LOGIN);
        return webcl;
    }

}
