/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web;

import java.io.IOException;
import java.util.List;
import java.util.Locale;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.HtmlAnchor;
import com.gargoylesoftware.htmlunit.html.HtmlPage;

import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.logging.Logger;

/***
 * Statische Implemtierungen der allg. Funktionen
 *
 * @author Frank Mnster
 */
public final class TargoBankWebSiteImpl {

    /**
     * ffnet die Kartenbersichtsseite
     *
     * @param webClient
     * @param accountInfo
     * @return die Webseite mit der Kreditkartenbersicht und den vorgemerkten Umstzen.
     * @throws WebSiteException
     *             falls irgendwas schief geht
     */
    public static HtmlPage openAccount(final WebClient webClient, final TargoBankWebAccountInformation accountInfo)
            throws WebSiteException {

        // Oeffne umsaetze
        HtmlPage pageKtoU;
        try {
            pageKtoU = webClient.getPage(TargoBank.URL_UEBERSICHT);
            if (pageKtoU.asText().contains("bersicht")) { //$NON-NLS-1$
                // Last 4 digits of the card number NOT of the account !!!
                final String linkName = Utils.maskCardNummer(accountInfo.getCardNumber()) + " " //$NON-NLS-1$
                        + accountInfo.getCardCompany();
                Logger.debug("LinkText (Karetnkonto): " + linkName); //$NON-NLS-1$
                final List<HtmlAnchor> anchors = pageKtoU.getAnchors();
                HtmlAnchor link = null;
                for (final HtmlAnchor htmlAnchor : anchors) {
                    Logger.trace(htmlAnchor.getTextContent());
                    if (htmlAnchor.getTextContent().equalsIgnoreCase(linkName)) {
                        Logger.debug("Anchor match!!!" //$NON-NLS-1$
                                + htmlAnchor.getTextContent().toUpperCase(Locale.GERMAN));
                        link = htmlAnchor;
                        break;
                    }
                }
                if (link == null) {
                    Logger.error(I18N.tr("TargoBankWebSiteImpl.cardNotFoundMsg") //$NON-NLS-1$
                            + linkName);
                    throw new WebSiteException(I18N.tr("TargoBankWebSiteImpl.cardNotFound"), //$NON-NLS-1$
                            WebSiteExceptionTypes.KONTO_NOT_FOUND);
                } else {
                    Logger.info(I18N.tr("TargoBankWebSiteImpl.cardFoundMsg", linkName)); //$NON-NLS-1$
                    final HtmlPage pageKreditDetails = link.click();
                    if (pageKreditDetails.asText().contains("Betrag der letzten Zahlung")) { //$NON-NLS-1$
                        // Kreditkartendetails offen --> zurueckgeben
                        Logger.debug("Details gefunden und offen. Nun zurueckgeben"); //$NON-NLS-1$
                        return pageKreditDetails;
                    } else {
                        Logger.error(I18N.tr("TargoBankWebSiteImpl.cardDetailsNotFound")); //$NON-NLS-1$
                        throw new WebSiteException(I18N.tr("TargoBankWebSiteImpl.cardDetailsNotFound"), //$NON-NLS-1$
                                WebSiteExceptionTypes.KONTO_NOT_FOUND);
                    }
                }
            } else {
                Logger.error(I18N.tr("TargoBankWebSiteImpl.openCardOverviewFailed")); //$NON-NLS-1$
                throw new WebSiteException(I18N.tr("TargoBankWebSiteImpl.openCardOverviewFailed"), //$NON-NLS-1$
                        WebSiteExceptionTypes.KONTO_NOT_FOUND);
            }
        } catch (FailingHttpStatusCodeException | IOException e) {
            final WebSiteException wse = new WebSiteException(e.getMessage(), WebSiteExceptionTypes.KONTO_NOT_FOUND);
            wse.addSuppressed(e);
            throw wse; // NOPMD
        }
    }

    /**
     * Privater Konstruktor
     */
    private TargoBankWebSiteImpl() {

        throw new AssertionError("Instantiating utility class..."); //$NON-NLS-1$
    }
}
