/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.gui.controller; // NOPMD

import java.rmi.RemoteException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.eclipse.swt.widgets.Event;

import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.frankmuenster.mahoe.banks.aktien.Aktien;
import de.frankmuenster.mahoe.banks.aktien.assistant.AktienCreateInfo;
import de.frankmuenster.mahoe.banks.aktien.gui.menu.AktienTabelleContextMenuListener;
import de.frankmuenster.mahoe.banks.aktien.gui.parts.AbstractStockSearchPart.SearchResultSelectListener;
import de.frankmuenster.mahoe.banks.aktien.gui.parts.AktienListEntry;
import de.frankmuenster.mahoe.banks.aktien.gui.parts.AktienSettingsAktienTabellePart;
import de.frankmuenster.mahoe.banks.aktien.gui.parts.BoerseArdSearchPart;
import de.frankmuenster.mahoe.banks.aktien.gui.parts.BoerseArdSelectExchangePart;
import de.frankmuenster.mahoe.banks.aktien.gui.parts.BoerseArdSelectExchangePart.ExchangeSelectListener;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.DecimalInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Mnster
 */
public class AktienSettingsController extends AbstractSettingsController<AktienCreateInfo> implements
        SearchResultSelectListener<ArdStockInfoBean>, ExchangeSelectListener, AktienTabelleContextMenuListener {

    /**
     * Platzhalter im Suchergebnis 10 Zeichen
     */
    protected static final String SPACE_10 = "          "; //$NON-NLS-1$

    /**
     * Platzhalter im Suchergebnis 20 Zeichen
     */
    protected static final String SPACE_21 = "                    "; //$NON-NLS-1$

    /**
     * Platzhalter im Suchergebnis 30 Zeichen
     */
    protected static final String SPACE_30 = "                              "; //$NON-NLS-1$

    /**
     * Platzhalter im Suchergebnis 3 Zeichen
     */
    protected static final String SPACE_3 = "   "; //$NON-NLS-1$

    /**
     *
     */
    public static final String FORMAT_ANZAHL = "###,###,##0.00####"; //$NON-NLS-1$

    /**
     * Update Button
     */
    private transient Button update;

    /**
     * Hinzufgen Button
     */
    private transient Button addStock;

    /**
     * Die ARD Wertpapier Infos
     */
    private transient ArdStockInfoBean bean;

    /**
     * Das Brsen-Auswahlfeld
     */
    private transient BoerseArdSelectExchangePart exchangesPart;

    /**
     * Das Ergebins Input
     */
    private transient DecimalInput stockAmount;

    /**
     * Das Wertpapier-Suchfeld
     */
    private transient BoerseArdSearchPart searchPart;

    /**
     * Die Ausgewhlte Brse mit der zugehrigen Wertpapier ID
     */
    private transient Pair<String, String> selectedExchange;

    /**
     * Listet alle Aktien des ausgwhlten Kontos
     */
    private transient AktienSettingsAktienTabellePart aktienTable;

    /**
     * Standardkonstruktor
     */
    public AktienSettingsController() {
        super(Aktien.BLZ, null);
        setDisplayAccountStatement(false);
        this.loadStocksfromAccount();
    }

    /**
     * @see ExchangeSelectListener#exchangeSelected(Pair)
     */
    @Override
    public void exchangeSelected(final Pair<String, String> exchange) {

        Logger.debug("Brsenplatz ausgewhlt: " + exchange.getKey() + ", ID: " + exchange.getValue()); //$NON-NLS-1$ //$NON-NLS-2$
        this.selectedExchange = exchange;
        this.updateAddButton();
    }

    /**
     * Liefert den Hinzufgen-Button
     *
     * @return der Hinzufgen-Button
     */
    public Button getAddStock() {

        if (this.addStock == null) {
            this.addStock = new Button(I18N.tr("AktienSettingsController.add"), new Action() { //$NON-NLS-1$

                /**
                 * @See de.willuhn.jameica.gui.Action#handleAction(Object)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    AktienSettingsController.this.handleAddStock();
                }
            });
        }
        this.updateAddButton();
        return this.addStock;
    }

    /**
     * Liefert eine Tabelle mit den schon existierenden Wertpapieren
     *
     * @return eine Tabelle mit den schon existierenden Wertpapieren
     */
    public AktienSettingsAktienTabellePart getAktienTable() {

        if (this.aktienTable == null) {
            final List<AktienListEntry> list = new ArrayList<>();
            this.aktienTable = new AktienSettingsAktienTabellePart(list, null, this);
            this.loadStocksfromAccount();
        }
        return this.aktienTable;
    }

    /**
     * Liefert das SearchPart
     *
     * @return das SearchPart
     */
    public BoerseArdSearchPart getSearchPart() {

        if (this.searchPart == null) {
            this.searchPart = new BoerseArdSearchPart(this, 10);
        }
        return this.searchPart;
    }

    /**
     * Liefert eine Liste mit allen zum WP gehrenden Brsenpltzen
     *
     * @return das exchnageListPart
     */
    public BoerseArdSelectExchangePart getSelectExchangePart() {

        if (this.exchangesPart == null) {
            this.exchangesPart = new BoerseArdSelectExchangePart(this);
        }
        return this.exchangesPart;
    }

    /**
     * Liefert eine TextInput fr die Ausgabe
     *
     * @return resultInput fr die Ausgabe
     * @throws RemoteException
     */
    public DecimalInput getStockAmountInput() throws RemoteException {

        if (this.stockAmount == null) {
            this.stockAmount = new DecimalInput(new DecimalFormat(AktienSettingsController.FORMAT_ANZAHL));
            this.stockAmount.setMandatory(true);
        }
        return this.stockAmount;
    }

    /**
     * An update Button only active/visible in case stock info has been loaded
     *
     * @return a button to update the selected stock
     */
    public Button getUpdateStock() {

        if (this.update == null) {
            this.update = new Button(I18N.tr("_update"), new Action() { //$NON-NLS-1$

                /**
                 * @See de.willuhn.jameica.gui.Action#handleAction(Object)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    AktienSettingsController.this.handleUpdateStock();
                }
            });
        }
        this.update.setEnabled(false);
        return this.update;
    }

    /**
     * @see AktienTabelleContextMenuListener#notifyStockToUpdateSelected(AktienListEntry)
     */
    @Override
    public void notifyStockToUpdateSelected(final AktienListEntry stock) {

        Logger.debug("Update stock ID: " + stock.getStockId()); //$NON-NLS-1$
        this.bean = new ArdStockInfoBean(stock.getStockId());
        try {
            this.bean = BoerseArdApi.getInstance().getActualValue(this.bean);
            // Update searchPart
            if (this.searchPart != null) {
                this.searchPart.setSearchText(this.bean.getName(), false);
            }
            // Update exchanges list
            if (this.exchangesPart != null) {
                final Map<String, String> exchanges = this.bean.getExchanges();
                this.exchangesPart.fillList(exchanges);
                this.exchangesPart.selectExchange(stock.getExchange());
            }
            // Update amount
            if (this.stockAmount != null) {
                final Double amount = Utils.convertStringToDouble(stock.getAmount());
                this.stockAmount.setValue(amount);
            }
            // finally activate update button
            if (this.update != null) {
                this.update.setEnabled(true);
            }
        } catch (final FinanceException e) {
            // notify user
            GUI.getStatusBar()
                    .setErrorText(I18N.tr("AktienSettingsController.errorLoadStockIdValues") + stock.getStockId()); //$NON-NLS-1$

        }
    }

    /**
     * @see de.frankmuenster.mahoe.banks.aktien.gui.parts.AbstractStockSearchPart.SearchResultSelectListener#searchResultSelected(de.frankmuenster.jameica.finanzen.StockInfoBean)
     */
    @Override
    public void searchResultSelected(final ArdStockInfoBean stockInfo) {

        this.bean = stockInfo;
        // Brsenpltze aktualisiern
        this.updateExchangeList();
        // Ausgewhlter Bresenplatz zurcksetzten
        this.selectedExchange = null; // NOPMD
        // Hinzufgen Button (de)aktivieren
        this.updateAddButton();
    }

    /**
     * @see de.frankmuenster.mahoe.gui.controller.SettingsController#speichern()
     */
    @Override
    public void speichern() {

        // Noch gibt es keine Einstellungen. Vielleicht mal spter.
    }

    /**
     * @see AbstractSettingsController#handleAccountChanged(org.eclipse.swt.widgets.Event)
     */
    @Override
    protected void handleAccountChanged(final Event event) {

        this.loadStocksfromAccount();
    }

    /**
     * Update the stock with the actual values.
     *
     * @throws ApplicationException
     */
    protected void handleUpdateStock() throws ApplicationException {

        if (this.bean != null) {

            final Konto kto = (Konto) this.getKonto().getValue();

            try {
                @SuppressWarnings("unchecked")
                final DBIterator<Umsatz> umsaetze = kto.getUmsaetze();
                Umsatz umsatz = null;
                while (umsaetze.hasNext()) {
                    umsatz = umsaetze.next();
                    if (this.bean.getId().equals(umsatz.getMeta(Aktien.STOCK_ID, StringUtils.EMPTY))) {
                        break;
                    }
                }
                if (umsatz == null) {
                    final String msg = I18N.tr("AktienSettingsController.stockToUpdateNotFound"); // NOPMD //$NON-NLS-1$
                    Logger.info(msg);
                    GUI.getStatusBar().setErrorText(msg);
                } else {
                    // Update the stock with the actual values
                    umsatz.setGegenkontoName(this.bean.getName());
                    umsatz.setArt(this.bean.getType());
                    final DecimalFormat dmf = new DecimalFormat(AktienSettingsController.FORMAT_ANZAHL);
                    umsatz.setZweck(I18N.tr("AktienSettingsController.amount") //$NON-NLS-1$
                            + dmf.format(this.getStockAmountInput().getValue()));
                    umsatz.setKommentar(I18N.tr("AktienSettingsController.exchange") + this.selectedExchange.getKey()); //$NON-NLS-1$
                    umsatz.store();

                    umsatz.setMeta(Aktien.STOCK_ID, this.selectedExchange.getValue());
                    umsatz.setMeta(Aktien.STOCK_EXCHANGE, this.selectedExchange.getKey());
                    final StatusBarMessage msg = new StatusBarMessage(I18N.tr("AktienSettingsController.stockUpdated"), //$NON-NLS-1$
                            StatusBarMessage.TYPE_SUCCESS);
                    Logger.info(msg.getText());
                    Application.getMessagingFactory().sendMessage(msg);
                    this.loadStocksfromAccount();
                }
            } catch (final RemoteException e) {
                throw new ApplicationException(e);
            }
        }

    }

    private void handleAddStock() throws ApplicationException {

        final Konto kto = (Konto) this.getKonto().getValue();
        HBCIDBService dbService;
        try {
            dbService = Settings.getDBService();
            final Umsatz umsatz = dbService.createObject(Umsatz.class, null);
            umsatz.setKonto(kto);
            umsatz.setGegenkontoName(this.bean.getName());
            umsatz.setArt(this.bean.getType());
            final DecimalFormat dmf = new DecimalFormat(AktienSettingsController.FORMAT_ANZAHL);
            umsatz.setZweck(
                    I18N.tr("AktienSettingsController.amount") + dmf.format(this.getStockAmountInput().getValue())); //$NON-NLS-1$
            umsatz.setKommentar(I18N.tr("AktienSettingsController.exchange") + this.selectedExchange.getKey()); //$NON-NLS-1$
            umsatz.setDatum(new Date());
            umsatz.setValuta(new Date());
            umsatz.setBetrag(0.0d);
            umsatz.setSaldo(0.0d);
            umsatz.store();
            umsatz.setMeta(Aktien.STOCK_ID, this.selectedExchange.getValue());
            umsatz.setMeta(Aktien.STOCK_EXCHANGE, this.selectedExchange.getKey());
            final StatusBarMessage msg = new StatusBarMessage(I18N.tr("AktienSettingsController.stockAddedMsg"), //$NON-NLS-1$
                    StatusBarMessage.TYPE_SUCCESS);
            Logger.info(msg.getText());
            Application.getMessagingFactory().sendMessage(msg);
            this.loadStocksfromAccount();
        } catch (final RemoteException e) {
            throw new ApplicationException(e);
        }
    }

    /**
     * Liest die vorhandenen Wertpapier
     */
    private void loadStocksfromAccount() {

        final ArrayList<AktienListEntry> aktienList = new ArrayList<>();
        final Konto kto = (Konto) this.getKonto().getValue();
        try {
            if (kto != null) {
                @SuppressWarnings("unchecked")
                final DBIterator<Umsatz> aktien = kto.getUmsaetze();
                while (aktien.hasNext()) {
                    final Umsatz stock = aktien.next();
                    String amount = "0.0"; //$NON-NLS-1$
                    final String[] zweck = stock.getZweck().split("="); //$NON-NLS-1$
                    if (zweck.length == 2 && StringUtils.isNotEmpty(zweck[1])) {
                        amount = zweck[1];
                    }
                    final String name = stock.getGegenkontoName();
                    final String exchange = stock.getMeta(Aktien.STOCK_EXCHANGE, StringUtils.EMPTY);
                    final String stockId = stock.getMeta(Aktien.STOCK_ID, StringUtils.EMPTY);
                    aktienList.add(new AktienListEntry(name, exchange, amount, stockId)); // NOPMD
                }
            }
            if (this.aktienTable != null) {
                this.aktienTable.removeAll();
                for (final AktienListEntry aktienListEntry : aktienList) {
                    this.aktienTable.addItem(aktienListEntry);
                }
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("AktienSettingsController.errorGettingExistingStocks") + e.getLocalizedMessage(), e); //$NON-NLS-1$
        }
    }

    /**
     * (De)Aktiviert den Brsenauswahl-Button
     */
    private void updateAddButton() {

        if (this.addStock != null) {
            if (this.selectedExchange == null) {
                this.addStock.setEnabled(false);
            } else {
                this.addStock.setEnabled(true);
            }
        }
    }

    /**
     * Zeigt die Brsenliste des aktuellen Wertpapieres an
     */
    private void updateExchangeList() {

        try {
            BoerseArdApi.getInstance().getActualValue(this.bean);
            if (this.exchangesPart != null) {
                this.exchangesPart.fillList(this.bean.getExchanges());
            }
        } catch (final FinanceException e) {
            Logger.error(I18N.tr("AktienSettingsController.errorGettingStockExchanges") + e.getLocalizedMessage(), e); //$NON-NLS-1$
        }
    }
}
